#include "Talkthrough.h"

//unsigned short Tx_Desc1[8], Tx_Desc2[8], Tx_Desc3[8];

short int Tx_Desc1[9], Tx_Desc2[9], Tx_Desc3[9];
short int Rx_Desc1[9], Rx_Desc2[9], Rx_Desc3[9];

section ("sdram0")int Rx_Buffer_A[Size];
section ("sdram0")int Rx_Buffer_B[Size];
section ("sdram0")int Rx_Buffer_C[Size];

section ("sdram0")short int Tx_Buffer_A[2*Size];
section ("sdram0")short int Tx_Buffer_B[2*Size];
section ("sdram0")short int Tx_Buffer_C[2*Size];

//--------------------------------------------------------------------------//
// Function:	Init_EBIU													//
//																			//
// Description:	This function initializes and enables asynchronous memory 	//
//				banks in External Bus Interface Unit so that Flash A can be //
//				accessed.													//
//--------------------------------------------------------------------------//
void Init_EBIU(void)
{
	*pEBIU_AMBCTL0	= 0x7bb07bb0;
	*pEBIU_AMBCTL1	= 0x7bb07bb0;
	*pEBIU_AMGCTL	= 0x000f;
}


//--------------------------------------------------------------------------//
// Function:	Init_Flash													//
//																			//
// Description:	This function initializes pin direction of Port A in Flash A//
//				to output.  The AD1836_RESET on the ADSP-BF533 EZ-KIT board //
//				is connected to Port A.										//
//--------------------------------------------------------------------------//
void Init_Flash(void)
{
	*pFlashA_PortA_Dir = 0x1;
}


//--------------------------------------------------------------------------//
// Function:	Init1836()													//
//																			//
// Description:	This function sets up the SPI port to configure the AD1836. //
//				The content of the array sCodec1836TxRegs is sent to the 	//
//				codec.														//
//--------------------------------------------------------------------------//
void Init1836(void)
{
int i;
	int j;
	static unsigned char ucActive_LED = 0x01;
	
	// write to Port A to reset AD1836
	*pFlashA_PortA_Data = 0x00;
	
	// write to Port A to enable AD1836
	*pFlashA_PortA_Data = ucActive_LED;
	
	// wait to recover from reset
	for (i=0; i<0xf0000; i++) asm("nop;");

	// Enable PF4
	*pSPI_FLG = FLS4;
	// Set baud rate SCK = HCLK/(2*SPIBAUD) SCK = 2MHz	
	*pSPI_BAUD = 16;
	// configure spi port
	// SPI DMA write, 16-bit data, MSB first, SPI Master
	*pSPI_CTL = TIMOD_DMA_TX | SIZE | MSTR;
	
	// Set up DMA5 to transmit
	// Map DMA5 to SPI
	*pDMA5_PERIPHERAL_MAP	= 0x5000;
	
	// Configure DMA5
	// 16-bit transfers
	*pDMA5_CONFIG = WDSIZE_16;
	// Start address of data buffer
	*pDMA5_START_ADDR = (void *)sCodec1836TxRegs;
	// DMA inner loop count
	*pDMA5_X_COUNT = CODEC_1836_REGS_LENGTH;
	// Inner loop address increment
	*pDMA5_X_MODIFY = 2;
	
	// enable DMAs
	*pDMA5_CONFIG = (*pDMA5_CONFIG | DMAEN);
	// enable spi
	*pSPI_CTL = (*pSPI_CTL | SPE);
	
	// wait until dma transfers for spi are finished 
	for (j=0; j<0xaff0; j++) asm("nop;");
	
	// disable spi
	*pSPI_CTL = 0x0000;
}


//--------------------------------------------------------------------------//
// Function:	Init_Sport0													//
//																			//
// Description:	Configure Sport0 for I2S mode Tx and SPORT1 for I2s mode Rx
//              external clock, external FS												//
//--------------------------------------------------------------------------//
void Init_Sport0(void)
{
	*pSPORT1_RFSDIV = 0xf;
	
	*pSPORT1_RCR2 = 0xf;
	
	*pSPORT1_RCR2 |= RSFSE;
	
	*pSPORT1_RCR1 = RCKFE;//|LARFS;

	*pSPORT0_TCR1 = TFSR;
	*pSPORT0_TCR2 = SLEN_16 | TSFSE;
}


//--------------------------------------------------------------------------//
// Function:	Init_DMA													//
//																			//
// Description:	3 descriptor models for Rx and Tx DMAs									//
//--------------------------------------------------------------------------//
void Init_DMA(void)
{
	
     Tx_Desc1[0] = ((int)Tx_Desc2 & 0xffff) ;
	Tx_Desc1[1] = ((int)Tx_Buffer_A & 0xffff) ;
	Tx_Desc1[2] = ((int)((int)Tx_Buffer_A >> 16) & 0xffff) ;
	Tx_Desc1[3] = FLOW_SMALL | NDSIZE_8 | WDSIZE_32 | DMAEN;//|DI_EN;
	Tx_Desc1[4] = Size/2;
	Tx_Desc1[5] = 4;
	Tx_Desc1[6] = 0;
	Tx_Desc1[7] = 0;
	
	Tx_Desc2[0] = ((int)Tx_Desc3 & 0xffff) ;
	Tx_Desc2[1] = ((int)Tx_Buffer_B & 0xffff) ;
	Tx_Desc2[2] = ((int)((int)Tx_Buffer_B >> 16) & 0xffff) ;
	Tx_Desc2[3] = FLOW_SMALL | NDSIZE_8 | WDSIZE_32 | DMAEN;//|DI_EN;
	Tx_Desc2[4] = Size/2;
	Tx_Desc2[5] = 4;
	Tx_Desc2[6] = 0;
	Tx_Desc2[7] = 0;
	
	
	Tx_Desc3[0] = ((int)Tx_Desc1 & 0xffff) ;
	Tx_Desc3[1] = ((int)Tx_Buffer_C & 0xffff) ;
	Tx_Desc3[2] = ((int)((int)Tx_Buffer_C >> 16) & 0xffff) ;
	Tx_Desc3[3] = FLOW_SMALL | NDSIZE_8 | WDSIZE_32 | DMAEN;//|DI_EN;
	Tx_Desc3[4] = Size/2;
	Tx_Desc3[5] = 4;
	Tx_Desc3[6] = 0;
	Tx_Desc3[7] = 0;
	
	//Info of Rx Descriptors
	
Rx_Desc1[0] = ((int)Rx_Desc2 & 0xffff) ;
	Rx_Desc1[1] = ((int)Rx_Buffer_A & 0xffff) ;
	Rx_Desc1[2] = ((int)((int)Rx_Buffer_A >> 16) & 0xffff) ;
	Rx_Desc1[3] = FLOW_SMALL | NDSIZE_8 | WDSIZE_32 | DI_EN | WNR | DMAEN;
	Rx_Desc1[4] = Size;
	Rx_Desc1[5] = 4;
	Rx_Desc1[6] = 0;
	Rx_Desc1[7] = 0;
	
	Rx_Desc2[0] = ((int)Rx_Desc3 & 0xffff) ;
	Rx_Desc2[1] = ((int)Rx_Buffer_B & 0xffff) ;
	Rx_Desc2[2] = ((int)((int)Rx_Buffer_B >> 16) & 0xffff) ;
	Rx_Desc2[3] = FLOW_SMALL | NDSIZE_8 | WDSIZE_32 | DI_EN | WNR | DMAEN;
	Rx_Desc2[4] = Size;
	Rx_Desc2[5] = 4;
	Rx_Desc2[6] = 0;
	Rx_Desc2[7] = 0;
	
	
	Rx_Desc3[0] = ((int)Rx_Desc1 & 0xffff) ;
	Rx_Desc3[1] = ((int)Rx_Buffer_C & 0xffff) ;
	Rx_Desc3[2] = ((int)((int)Rx_Buffer_C >> 16) & 0xffff) ;
	Rx_Desc3[3] = FLOW_SMALL | NDSIZE_8 | WDSIZE_32 | DI_EN | WNR |  DMAEN;
	Rx_Desc3[4] = Size;
	Rx_Desc3[5] = 4;
	Rx_Desc3[6] = 0;
	Rx_Desc3[7] = 0;
	
	// Start the DMA by writing to NEXT DESC. PTR REGISTER
	(*pDMA3_NEXT_DESC_PTR) = (int *)Rx_Desc1 ;
	ssync();
	(*pDMA2_NEXT_DESC_PTR) = (int *)Tx_Desc3 ;
	ssync();
	// Now configure the DMA config register finally
	(*pDMA3_CONFIG) = FLOW_SMALL | NDSIZE_8 | WDSIZE_32 | DI_EN | WNR; 
	(*pDMA2_CONFIG) = FLOW_SMALL | NDSIZE_8 | WDSIZE_32;//DI_EN|

	
}
//--------------------------------------------------------------------------//
// Function:	Enable_DMA_Sport											//
//																			//
// Description:	Enable DMA3, Sport0 RX					//
//--------------------------------------------------------------------------//
void Enable_DMA_Sport0(void)
{   *pDMA3_CONFIG	= (*pDMA3_CONFIG | DMAEN);

	*pSPORT1_RCR1 	= (*pSPORT1_RCR1 | RSPEN);
}

//--------------------------------------------------------------------------//
// Function:	Init_Interrupts												//
//																			//
// Description:	Initialize Interrupt for Sport0 RX							//
//--------------------------------------------------------------------------//
void Init_Interrupts(void)
{
	// Set Sport0 RX (DMA3) interrupt priority to 2 = IVG9
	*pSIC_IAR0 = 0xffffffff;
	*pSIC_IAR1 = 0xffff23ff;
	*pSIC_IAR2 = 0xffffffff;

	// assign ISRs to interrupt vectors
	// Sport0 RX ISR -> IVG 9
	register_handler(ik_ivg9, Sport1_RX_ISR);
	register_handler(ik_ivg10, Sport0_TX_ISR);
	// enable Sport0 TX interrupt
	*pSIC_IMASK = 0x00000c00;
}
